"""
Package boxen allows you to create customizable boxes in the terminal

It is built on top of Rich, and is fully compatible with most Rich styling syntax and Rich renderables
"""

__version__ = "1.1.1"

from .main import *
